@@ -39,6 +39,10 @@
  *
  */
 
+/**
+ * @file
+ */
+
 #include "common/common_types.h"
 #include "common/netaddr.h"
 #include "core/oonf_logging.h"
@@ -83,9 +87,9 @@ static void _cleanup_error(void);
 static enum rfc5444_result _pass2_process_localif(struct netaddr *addr, uint8_t local_if);
 static void _handle_originator(struct rfc5444_reader_tlvblock_context *context);
 
-static enum rfc5444_result
-_cb_messagetlvs(struct rfc5444_reader_tlvblock_context *context);
-enum rfc5444_result _cb_failed_constraints(
+static enum rfc5444_result _cb_messagetlvs(
+    struct rfc5444_reader_tlvblock_context *context);
+static enum rfc5444_result _cb_failed_constraints(
       struct rfc5444_reader_tlvblock_context *context);
 
 static enum rfc5444_result
@@ -101,7 +105,7 @@ static enum rfc5444_result _cb_msg_pass2_end(
 /* definition of the RFC5444 reader components */
 static struct rfc5444_reader_tlvblock_consumer _nhdp_message_pass1_consumer = {
   .order = RFC5444_MAIN_PARSER_PRIORITY,
-  .msg_id = RFC5444_MSGTYPE_HELLO,
+  .msg_id = RFC6130_MSGTYPE_HELLO,
   .block_callback = _cb_messagetlvs,
   .block_callback_failed_constraints = _cb_failed_constraints,
   .end_callback = _cb_addresstlvs_pass1_end,
@@ -125,7 +129,7 @@ static struct rfc5444_reader_tlvblock_consumer_entry _nhdp_message_tlvs[] = {
 
 static struct rfc5444_reader_tlvblock_consumer _nhdp_address_pass1_consumer = {
   .order = RFC5444_MAIN_PARSER_PRIORITY,
-  .msg_id = RFC5444_MSGTYPE_HELLO,
+  .msg_id = RFC6130_MSGTYPE_HELLO,
   .addrblock_consumer = true,
   .block_callback = _cb_addresstlvs_pass1,
   .block_callback_failed_constraints = _cb_failed_constraints,
@@ -140,14 +144,14 @@ static struct rfc5444_reader_tlvblock_consumer_entry _nhdp_address_pass1_tlvs[]
 
 static struct rfc5444_reader_tlvblock_consumer _nhdp_message_pass2_consumer = {
   .order = RFC5444_MAIN_PARSER_PRIORITY + 1,
-  .msg_id = RFC5444_MSGTYPE_HELLO,
+  .msg_id = RFC6130_MSGTYPE_HELLO,
   .end_callback = _cb_msg_pass2_end,
   .block_callback_failed_constraints = _cb_failed_constraints,
 };
 
 static struct rfc5444_reader_tlvblock_consumer _nhdp_address_pass2_consumer= {
   .order = RFC5444_MAIN_PARSER_PRIORITY + 1,
-  .msg_id = RFC5444_MSGTYPE_HELLO,
+  .msg_id = RFC6130_MSGTYPE_HELLO,
   .addrblock_consumer = true,
   .block_callback = _cb_addr_pass2_block,
   .block_callback_failed_constraints = _cb_failed_constraints,
@@ -191,6 +195,7 @@ static struct {
 
 /**
  * Initialize nhdp reader
+ * @param p rfc5444 protocol
  */
 void
 nhdp_reader_init(struct oonf_rfc5444_protocol *p) {
@@ -378,13 +383,8 @@ _cb_messagetlvs(struct rfc5444_reader_tlvblock_context *context) {
 
   OONF_INFO(LOG_NHDP_R,
       "Incoming message type %d from %s through %s (addrlen = %u), got message tlvs",
-      context->msg_type, netaddr_socket_to_string(&buf, _protocol->input_socket),
-      _protocol->input_interface->name, context->addr_len);
-
-  if (!_protocol->input_is_multicast) {
-    /* NHDP doesn't care about unicast messages */
-    return RFC5444_DROP_MESSAGE;
-  }
+      context->msg_type, netaddr_socket_to_string(&buf, _protocol->input.src_socket),
+      _protocol->input.interface->name, context->addr_len);
 
   switch (context->addr_len) {
     case 4:
@@ -398,14 +398,18 @@ _cb_messagetlvs(struct rfc5444_reader_tlvblock_context *context) {
       break;
   }
 
-  if (!oonf_rfc5444_is_interface_active(_protocol->input_interface, af_type)) {
+  if (!oonf_rfc5444_is_interface_active(_protocol->input.interface, af_type)) {
     OONF_DEBUG(LOG_NHDP_R, "We do not handle address length %u on interface %s",
-        context->addr_len, _protocol->input_interface->name);
+        context->addr_len, _protocol->input.interface->name);
     return RFC5444_DROP_MESSAGE;
   }
 
   /* remember local NHDP interface */
-  _current.localif = nhdp_interface_get(_protocol->input_interface->name);
+  _current.localif = nhdp_interface_get(_protocol->input.interface->name);
+  if (!_current.localif) {
+    /* incoming message through an interface unspecific socket, ignore it */
+    return RFC5444_DROP_MESSAGE;
+  }
 
   /* extract originator address */
   if (context->has_origaddr) {
@@ -467,7 +471,7 @@ _cb_messagetlvs(struct rfc5444_reader_tlvblock_context *context) {
   return RFC5444_OKAY;
 }
 
-enum rfc5444_result
+static enum rfc5444_result
 _cb_failed_constraints(struct rfc5444_reader_tlvblock_context *context __attribute__((unused))) {
 #ifdef OONF_LOG_INFO
   struct netaddr_str nbuf;
@@ -475,8 +479,8 @@ _cb_failed_constraints(struct rfc5444_reader_tlvblock_context *context __attribu
 
   OONF_INFO(LOG_NHDP_R,
       "Incoming message type %d from %s through %s (addrlen = %u) failed constraints",
-      context->msg_type, netaddr_socket_to_string(&nbuf, _protocol->input_socket),
-      _protocol->input_interface->name, context->addr_len);
+      context->msg_type, netaddr_socket_to_string(&nbuf, _protocol->input.src_socket),
+      _protocol->input.interface->name, context->addr_len);
   return RFC5444_DROP_MESSAGE;
 }
 
@@ -640,7 +644,7 @@ _cb_addresstlvs_pass1_end(struct rfc5444_reader_tlvblock_context *context, bool
   }
 
   /* copy interface address of link */
-  memcpy(&_current.link->if_addr, _protocol->input_address, sizeof(struct netaddr));
+  memcpy(&_current.link->if_addr, _protocol->input.src_address, sizeof(struct netaddr));
 
   /* copy mac address */
   if (netaddr_get_address_family(&_current.mac) == AF_MAC48) {
@@ -651,8 +655,8 @@ _cb_addresstlvs_pass1_end(struct rfc5444_reader_tlvblock_context *context, bool
     struct netaddr addr;
 
     /* translate like a RFC5444 address */
-    if(netaddr_from_binary(&addr, netaddr_get_binptr(_protocol->input_address),
-        netaddr_get_binlength(_protocol->input_address), 0)) {
+    if(netaddr_from_binary(&addr, netaddr_get_binptr(_protocol->input.src_address),
+        netaddr_get_binlength(_protocol->input.src_address), 0)) {
       return RFC5444_DROP_MESSAGE;
     }
 
