@@ -57,6 +57,7 @@
 #include "subsystems/oonf_rfc5444.h"
 #include "subsystems/oonf_telnet.h"
 #include "subsystems/oonf_timer.h"
+#include "subsystems/os_interface.h"
 
 #include "nhdp/nhdp_interfaces.h"
 
@@ -109,8 +110,11 @@ struct _config {
   /*! olsrv2 p_hold_time */
   uint64_t p_hold_time;
 
+  /*! decides NHDP routable status */
+  bool nhdp_routable;
+
   /*! IP filter for routable addresses */
-  struct netaddr_acl routable;
+  struct netaddr_acl routable_acl;
 
   /*! IP filter for valid originator */
   struct netaddr_acl originator_acl;
@@ -120,8 +124,8 @@ struct _config {
  * Additional parameters of a single locally attached network
  */
 struct _lan_data {
-  /*! domain of LAN */
-  struct nhdp_domain *domain;
+  /*! extension domain of LAN */
+  int32_t ext;
 
   /*! source prefix */
   struct netaddr source_prefix;
@@ -145,7 +149,7 @@ static void _parse_lan_array(struct cfg_named_section *section, bool add);
 static void _cb_generate_tc(struct oonf_timer_instance *);
 
 static void _update_originator(int af_family);
-static int _cb_if_event(struct oonf_interface_listener *);
+static int _cb_if_event(struct os_interface_listener *);
 
 static void _cb_cfg_olsrv2_changed(void);
 static void _cb_cfg_domain_changed(void);
@@ -182,15 +186,19 @@ static struct cfg_schema_entry _olsrv2_entries[] = {
     "Holdtime for forwarding set information", 100),
   CFG_MAP_CLOCK_MIN(_config, p_hold_time, "processing_hold_time", "300.0",
     "Holdtime for processing set information", 100),
-  CFG_MAP_ACL_V46(_config, routable, "routable",
+  CFG_MAP_BOOL(_config, nhdp_routable, "nhdp_routable", "no",
+    "Decides if NHDP interface addresses"
+    " are routed to other nodes. 'true' means the 'routable_acl' parameter"
+    " will be matched to the addresses to decide."),
+  CFG_MAP_ACL_V46(_config, routable_acl, "routable_acl",
       OLSRV2_ROUTABLE_IPV4 OLSRV2_ROUTABLE_IPV6 ACL_DEFAULT_ACCEPT,
     "Filter to decide which addresses are considered routable"),
 
   CFG_VALIDATE_LAN(_LOCAL_ATTACHED_NETWORK_KEY, "",
     "locally attached network, a combination of an"
     " ip address or prefix followed by an up to four optional parameters"
-    " which define link metric cost, hopcount distance and domain of the prefix"
-    " ( <"LAN_OPTION_METRIC"...> <"LAN_OPTION_DIST"...>"
+    " which define link metric cost, hopcount distance, domain of the prefix"
+    " and the source-prefix ( <"LAN_OPTION_METRIC"...> <"LAN_OPTION_DIST"...>"
     " <"LAN_OPTION_DOMAIN"<num>/all> <"LAN_OPTION_SRC"...> ).",
     .list = true),
 
@@ -198,7 +206,7 @@ static struct cfg_schema_entry _olsrv2_entries[] = {
     OLSRV2_ORIGINATOR_IPV4 OLSRV2_ORIGINATOR_IPV6 ACL_DEFAULT_ACCEPT,
     "Filter for router originator addresses (ipv4 and ipv6)"
     " from the interface addresses. Olsrv2 will prefer routable addresses"
-    " over linklocal addresses."),
+    " over linklocal addresses and addresses from loopback over other interfaces."),
 };
 
 static struct cfg_schema_section _olsrv2_section = {
@@ -211,9 +219,9 @@ static struct cfg_schema_section _olsrv2_section = {
 
 static const char *_dependencies[] = {
   OONF_CLASS_SUBSYSTEM,
-  OONF_INTERFACE_SUBSYSTEM,
   OONF_RFC5444_SUBSYSTEM,
   OONF_TIMER_SUBSYSTEM,
+  OONF_OS_INTERFACE_SUBSYSTEM,
   OONF_NHDP_SUBSYSTEM,
 };
 static struct oonf_subsystem _olsrv2_subsystem = {
@@ -242,20 +250,24 @@ static struct oonf_timer_instance _tc_timer = {
 };
 
 /* global interface listener */
-static struct oonf_interface_listener _if_listener = {
-  .process = _cb_if_event,
+static struct os_interface_listener _if_listener = {
+  .name = OS_INTERFACE_ANY,
+  .if_changed = _cb_if_event,
 };
 
 /* global variables */
 static struct oonf_rfc5444_protocol *_protocol;
 
 static uint16_t _ansn;
+static uint32_t _sym_neighbor_id = 0;
+
+static bool _generate_tcs = true;
 
 /* Additional logging sources */
-enum oonf_log_source LOG_OLSRV2;
-enum oonf_log_source LOG_OLSRV2_R;
-enum oonf_log_source LOG_OLSRV2_ROUTING;
-enum oonf_log_source LOG_OLSRV2_W;
+static enum oonf_log_source LOG_OLSRV2;
+static enum oonf_log_source LOG_OLSRV2_R;
+static enum oonf_log_source LOG_OLSRV2_ROUTING;
+static enum oonf_log_source LOG_OLSRV2_W;
 
 /**
  * Initialize additional logging sources for NHDP
@@ -284,7 +296,7 @@ _init(void) {
   }
 
   /* activate interface listener */
-  oonf_interface_add_listener(&_if_listener);
+  os_interface_add(&_if_listener);
 
   /* activate the rest of the olsrv2 protocol */
   olsrv2_lan_init();
@@ -315,10 +327,10 @@ _initiate_shutdown(void) {
 static void
 _cleanup(void) {
   /* remove interface listener */
-  oonf_interface_remove_listener(&_if_listener);
+  os_interface_remove(&_if_listener);
 
   /* cleanup configuration */
-  netaddr_acl_remove(&_olsrv2_config.routable);
+  netaddr_acl_remove(&_olsrv2_config.routable_acl);
   netaddr_acl_remove(&_olsrv2_config.originator_acl);
 
   /* cleanup all parts of olsrv2 */
@@ -348,13 +360,25 @@ olsrv2_get_tc_validity(void) {
 }
 
 /**
- * @return acl for checking if an address is routable
+ * @param addr NHDP address to be checked
+ * @return true if address should be routed, false otherwise
  */
-const struct netaddr_acl *
-olsrv2_get_routable(void) {
-    return &_olsrv2_config.routable;
+bool
+olsrv2_is_nhdp_routable(struct netaddr *addr) {
+  if (!_olsrv2_config.nhdp_routable) {
+    return false;
+  }
+  return olsrv2_is_routable(addr);
 }
 
+/**
+ * @param addr address to be checked
+ * @return true if address should be routed, false otherwise
+ */
+bool
+olsrv2_is_routable(struct netaddr *addr) {
+  return netaddr_acl_check_accept(&_olsrv2_config.routable_acl, addr);
+}
 
 /**
  * default implementation for rfc5444 processing handling according
@@ -424,7 +448,7 @@ olsrv2_mpr_shall_forwarding(struct rfc5444_reader_tlvblock_context *context,
   }
 
   /* check input interface */
-  if (_protocol->input_interface == NULL) {
+  if (_protocol->input.interface == NULL) {
     OONF_DEBUG(LOG_OLSRV2, "Do not forward because input interface is not set");
     return false;
   }
@@ -436,7 +460,7 @@ olsrv2_mpr_shall_forwarding(struct rfc5444_reader_tlvblock_context *context,
   }
 
   /* check if this is coming from the unicast receiver */
-  if (strcmp(_protocol->input_interface->name, RFC5444_UNICAST_INTERFACE) == 0) {
+  if (strcmp(_protocol->input.interface->name, RFC5444_UNICAST_INTERFACE) == 0) {
     return false;
   }
 
@@ -454,10 +478,10 @@ olsrv2_mpr_shall_forwarding(struct rfc5444_reader_tlvblock_context *context,
   }
 
   /* get NHDP interface */
-  interf = nhdp_interface_get(_protocol->input_interface->name);
+  interf = nhdp_interface_get(_protocol->input.interface->name);
   if (interf == NULL) {
     OONF_DEBUG(LOG_OLSRV2, "Do not forward because NHDP does not handle"
-        " interface '%s'", _protocol->input_interface->name);
+        " interface '%s'", _protocol->input.interface->name);
     return false;
   }
 
@@ -508,10 +532,11 @@ olsrv2_get_ansn(void) {
 
 /**
  * Update answer set number if metric of a neighbor changed since last update.
+ * @param force true to force an answerset number change
  * @return new answer set number, might be the same if no metric changed.
  */
 uint16_t
-olsrv2_update_ansn(void) {
+olsrv2_update_ansn(bool force) {
   struct nhdp_domain *domain;
   bool changed;
 
@@ -523,12 +548,29 @@ olsrv2_update_ansn(void) {
     }
   }
 
-  if (changed) {
+  if (changed || force
+      || _sym_neighbor_id != nhdp_db_neighbor_get_set_id()) {
     _ansn++;
+    _sym_neighbor_id = nhdp_db_neighbor_get_set_id();
   }
   return _ansn;
 }
 
+/**
+ * Switches the automatic generation of TCs on and off
+ * @param generate true if TCs should be generated every OLSRv2 TC interval,
+ *   false otherwise
+ */
+void
+olsrv2_generate_tcs(bool generate) {
+  if (generate && !oonf_timer_is_active(&_tc_timer)) {
+    oonf_timer_set(&_tc_timer, _olsrv2_config.tc_interval);
+  }
+  else if (!generate && oonf_timer_is_active(&_tc_timer)) {
+    oonf_timer_stop(&_tc_timer);
+  }
+}
+
 /**
  * Schema entry validator for an attached network.
  * See CFG_VALIDATE_ACL_*() macros.
@@ -613,7 +655,7 @@ _parse_lan_parameters(struct os_route_key *prefix,
   unsigned ext;
 
   ptr = src;
-  dst->domain = NULL;
+  dst->ext = -1;
   dst->metric = LAN_DEFAULT_METRIC;
   dst->dist   = LAN_DEFAULT_DISTANCE;
 
@@ -627,19 +669,15 @@ _parse_lan_parameters(struct os_route_key *prefix,
       }
     }
     else if (strncasecmp(buffer, LAN_OPTION_DOMAIN, 7) == 0) {
-      fprintf(stderr, "domain='%s'\n", &buffer[7]);
       if (strcasecmp(&buffer[7], "all") == 0) {
-        dst->domain = NULL;
+        dst->ext = -1;
       }
       else {
         ext = strtoul(&buffer[7], NULL, 10);
         if ((ext == 0 && errno != 0) || ext > 255) {
           return "an illegal domain parameter";
         }
-        dst->domain = nhdp_domain_get_by_ext(ext);
-        if (dst->domain == NULL) {
-          return "an unknown domain extension number";
-        }
+        dst->ext = ext;
       }
     }
     else if (strncasecmp(buffer, LAN_OPTION_DIST, 5) == 0) {
@@ -682,6 +720,7 @@ _parse_lan_array(struct cfg_named_section *section, bool add) {
   struct netaddr addr;
   struct os_route_key prefix;
   struct _lan_data data;
+  struct nhdp_domain *domain;
 
   const char *value, *ptr;
   struct cfg_entry *entry;
@@ -711,22 +750,26 @@ _parse_lan_array(struct cfg_named_section *section, bool add) {
       continue;
     }
 
-    if (data.domain == NULL) {
-      list_for_each_element(nhdp_domain_get_list(), data.domain, _node) {
+    if (data.ext == -1) {
+      list_for_each_element(nhdp_domain_get_list(), domain, _node) {
         if (add) {
-          olsrv2_lan_add(data.domain, &prefix, data.metric, data.dist);
+          olsrv2_lan_add(domain, &prefix, data.metric, data.dist);
         }
         else {
-          olsrv2_lan_remove(data.domain, &prefix);
+          olsrv2_lan_remove(domain, &prefix);
         }
       }
     }
     else {
+      domain = nhdp_domain_add(data.ext);
+      if (!domain) {
+        continue;
+      }
       if (add) {
-        olsrv2_lan_add(data.domain, &prefix, data.metric, data.dist);
+        olsrv2_lan_add(domain, &prefix, data.metric, data.dist);
       }
       else {
-        olsrv2_lan_remove(data.domain, &prefix);
+        olsrv2_lan_remove(domain, &prefix);
       }
     }
   }
@@ -743,7 +786,7 @@ _cb_generate_tc(struct oonf_timer_instance *ptr __attribute__((unused))) {
   }
 }
 
-static int
+static uint32_t
 _get_addr_priority(const struct netaddr *addr) {
 #ifdef OONF_LOG_DEBUG_INFO
   struct netaddr_str nbuf;
@@ -760,29 +803,29 @@ _get_addr_priority(const struct netaddr *addr) {
   if (netaddr_get_address_family(addr) == AF_INET) {
     if (netaddr_is_in_subnet(&NETADDR_IPV4_LINKLOCAL, addr)) {
       /* linklocal */
-      OONF_DEBUG(LOG_OLSRV2, "check priority for %s: 2 (linklocal)",
+      OONF_DEBUG(LOG_OLSRV2, "check priority for %s: 1 (linklocal)",
           netaddr_to_string(&nbuf, addr));
-      return 2;
+      return 1;
     }
 
     /* routable */
-    OONF_DEBUG(LOG_OLSRV2, "check priority for %s: 4 (routable)",
+    OONF_DEBUG(LOG_OLSRV2, "check priority for %s: 2 (routable)",
         netaddr_to_string(&nbuf, addr));
-    return 4;
+    return 2;
   }
 
   if (netaddr_get_address_family(addr) == AF_INET6) {
     if (netaddr_is_in_subnet(&NETADDR_IPV6_LINKLOCAL, addr)) {
       /* linklocal */
-      OONF_DEBUG(LOG_OLSRV2, "check priority for %s: 2 (linklocal)",
+      OONF_DEBUG(LOG_OLSRV2, "check priority for %s: 1 (linklocal)",
           netaddr_to_string(&nbuf, addr));
-      return 2;
+      return 1;
     }
 
     /* routable */
-    OONF_DEBUG(LOG_OLSRV2, "check priority for %s: 4 (routable)",
+    OONF_DEBUG(LOG_OLSRV2, "check priority for %s: 2 (routable)",
         netaddr_to_string(&nbuf, addr));
-    return 4;
+    return 2;
   }
 
   /* unknown */
@@ -799,13 +842,13 @@ _get_addr_priority(const struct netaddr *addr) {
 static void
 _update_originator(int af_family) {
   const struct netaddr *originator;
-  struct nhdp_interface *n_interf;
-  struct os_interface *interf;
+  struct nhdp_interface *nhdp_if;
+  struct os_interface_listener *if_listener;
   struct netaddr new_originator;
-  int new_priority;
-  int old_priority;
-  int priority;
-  size_t i;
+  struct os_interface_ip *ip;
+  uint32_t new_priority;
+  uint32_t old_priority;
+  uint32_t priority;
 #ifdef OONF_LOG_DEBUG_INFO
   struct netaddr_str buf;
 #endif
@@ -820,26 +863,27 @@ _update_originator(int af_family) {
 
   netaddr_invalidate(&new_originator);
 
-  avl_for_each_element(nhdp_interface_get_tree(), n_interf, _node) {
-    interf = nhdp_interface_get_coreif(n_interf);
+  avl_for_each_element(nhdp_interface_get_tree(), nhdp_if, _node) {
+    if_listener = nhdp_interface_get_if_listener(nhdp_if);
 
     /* check if originator is still valid */
-    for (i=0; i<interf->data.addrcount; i++) {
-      struct netaddr *addr = &interf->data.addresses[i];
-
-      if (netaddr_get_address_family(addr) == af_family) {
-        priority = _get_addr_priority(addr);
+    avl_for_each_element(&if_listener->data->addresses, ip, _node) {
+      if (netaddr_get_address_family(&ip->address) == af_family) {
+        priority = _get_addr_priority(&ip->address) * 4;
         if (priority == 0) {
           /* not useful */
           continue;
         }
 
-        if (netaddr_cmp(originator, addr) == 0) {
+        if (if_listener->data->flags.loopback) {
+          priority += 2;
+        }
+        if (netaddr_cmp(originator, &ip->address) == 0) {
           old_priority = priority + 1;
         }
 
         if (priority > old_priority && priority > new_priority) {
-          memcpy(&new_originator, addr, sizeof(new_originator));
+          memcpy(&new_originator, &ip->address, sizeof(new_originator));
           new_priority = priority;
         }
       }
@@ -859,7 +903,7 @@ _update_originator(int af_family) {
  * @return always 0
  */
 static int
-_cb_if_event(struct oonf_interface_listener *listener __attribute__((unused))) {
+_cb_if_event(struct os_interface_listener *if_listener __attribute__((unused))) {
   _update_originator(AF_INET);
   _update_originator(AF_INET6);
   return 0;
@@ -877,7 +921,9 @@ _cb_cfg_olsrv2_changed(void) {
   }
 
   /* set tc timer interval */
-  oonf_timer_set(&_tc_timer, _olsrv2_config.tc_interval);
+  if (_generate_tcs) {
+    oonf_timer_set(&_tc_timer, _olsrv2_config.tc_interval);
+  }
 
   /* check if we have to change the originators */
   _update_originator(AF_INET);
